import os
import sys
import unittest

import numpy as np

from extended_train_test_split import train_test_split


class Test_extended_train_test_split(unittest.TestCase):
    """
    Test the various functionalities of extended_train_test_split.
    """

    @classmethod
    def setUpClass(self):
        return

    def test_train_test_split(self):
        """
        """
        X_train, X_test, y_train, y_test = train_test_split(
            np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]]),
            np.array([10, 11, 12]),
            test_size=0.2,
            train_size=0.8,
            splitter='random',
            hopts={
                'shuffle': True,
                'random_state': 42,
            }
        )
        for elt, ans in zip(X_train.flatten(), [4, 5, 6, 7, 8, 9]):
            self.assertEqual(elt, ans)


if __name__ == '__main__':
    unittest.main()



"""Abstract Sampling class"""
from abc import ABC, abstractmethod

from ..exceptions import *


class Sampler(abc):
    @abstractmethod
    def __init__(self, configs):
        pass

    @abstractmethod
    def populate(self, X, y=None):
        """
        Load data in this instance.

        """
        self.X = X
        self.y = y
        self.samples = []

    @abstractmethod
    def get_sample(self):
        """
        Get one sample.

        """
        if not self.is_populated:
            raise NotInitializedError('Populate sampler instance with '
                                      'data to get samples')
        if len(self.samples) == len(X):
            raise ValueError('Dataset exhausted')

        self.samples.append(self.get_next_samle_id())
        return self.samples[-1]

    @abstractmethod
    def get_next_sample_id(self):
        """
        Get the id of the next sample.

        """
        pass

    @abstractmethod
    def get_batch_samples(self, n_samples):
        """
        Get a batch of samples

        """
        return self.X[self.get_batch_sample_idx(n_samples)]

    @abstractmethod
    def get_batch_sample_idx(self, n_samples):
        """
        Get idx of the next batch of samples.

        """
        return [self.get_next_sample_id() for _ in range(n_samples)]



from sklearn.model_selection import train_test_split


def random(
    X,
    y=None,
    test_size=None,
    train_size=None,
    random_state=None,
    shuffle=True,
    stratify=None,
):
    return train_test_split(
        X,
        y,
        test_size=test_size,
        train_size=train_size,
        random_state=random_state,
        shuffle=shuffle,
        stratify=stratify,
    )



import numpy as np
from extended_train_test_split.samplers import (
    random,
    kennard_stone,
)


def train_test_split(
    X: np.array,
    y: np.array = None,
    test_size: float = 0.25,
    train_size: float = 0.75,
    splitter: str = 'random',
    hopts: dict = {},
):
    if splitter == 'random':
        return random(
            X,
            y=y,
            test_size=test_size,
            train_size=train_size,
            shuffle=hopts.get("shuffle", False),
            random_state=hopts.get("random_state", None),
            stratify=hopts.get("stratify", None),
        )
    elif splitter == 'kennard_stone':
        return kennard_stone(
            X,
            y=y,
            test_size=test_size,
            train_size=train_size,
        )
    else:
        raise NotImplementedError(
            f'Splitter "{splitter}" has not been implemented.'
        )



import os
import sys
import unittest

import numpy as np

from extended_train_test_split.interfaces import train_test_split_molecules


class Test_molecules_interface(unittest.TestCase):
    """
    Test the various functionalities of molecules_interface.
    """

    @classmethod
    def setUpClass(self):
        return

    def test_molecules_interface(self):
        """
        """
        return


if __name__ == '__main__':
    unittest.main()



from typing import List
import numpy as np
from AIMSim.chemical_datastructures import Molecule

from extended_train_test_split import train_test_split


def train_test_split_molecules(
    smiles: List[str],
    y: np.array = None,
    test_size: float = 0.25,
    train_size: float = 0.75,
    splitter: str = 'random',
    fingerprint: str = 'morgan_fingerprint',
    return_as: str = 'fprint',
    hopts: dict = {},
    fprints_hopts: dict = {},
):
    # turn the smiles into an input X
    X = []
    for smile in smiles:
        mol = Molecule(mol_smiles=smile)
        mol.descriptor.make_fingerprint(
            mol.mol_graph,
            fingerprint,
            fingerprint_params=fprints_hopts,
        )
        X.append(mol.descriptor.to_numpy())
    # call train test split with this input
    return train_test_split(
        X,
        y=y,
        test_size=test_size,
        train_size=train_size,
        splitter=splitter,
        hopts=hopts,
    )



import os
import sys
import unittest

import numpy as np

from extended_train_test_split.samplers import kennard_stone


class Test_kennard_stone(unittest.TestCase):
    """
    Test the various functionalities of kennard_stone.
    """

    @classmethod
    def setUpClass(self):
        return

    def test_kennard_stone(self):
        """
        """
        return


if __name__ == '__main__':
    unittest.main()



from kennard_stone import train_test_split


def kennard_stone(
    X,
    y=None,
    test_size=None,
    train_size=None,
):
    return train_test_split(
        X,
        y,
        test_size=test_size,
        train_size=train_size,
    )



import os.path
import codecs
import pathlib
from setuptools import setup, find_packages


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


# The directory containing this file
cwd = pathlib.Path(__file__).parent

# The text of the README file
README = (cwd / "README.md").read_text()

setup(
    name="astartes",
    version=get_version("extended_train_test_split/__init__.py"),
    description="Algorithmic train:test splitting for molecules, images, and arbitrary arrays.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/JacksonBurns/extended_train_test_split",
    author="Jackson Burns",
    author_email="jburnsky@udel.edu",
    license="MIT",
    classifiers=["Programming Language :: Python :: 3"],
    install_requires=read("requirements.txt").split("\n"),
    packages=find_packages(),
    include_package_data=True,
)
